# Generated by Django 4.1.4 on 2023-02-03 06:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0031_remove_timestampedmodel_because_wagtail"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="blog",
            name="created",
        ),
        migrations.RemoveField(
            model_name="blog",
            name="modified",
        ),
    ]
