{% load wagtailcore_tags %}

<article>
  <header>
    <h2><a class="text-dark text-decoration-none" href="{% pageurl page %}">{{ page.title }}</a></h2>
    <!-- link is on one line to avoid underlined whitespace -->
    <a href="{% pageurl page %}"><time date-time='{{ page.visible_date|date:"c" }}'>{{ page.visible_date|date }}</time>,</a>
    <span class="author">{{ page.owner.username | title}}</span>
  </header>
  {% for block in page.body %}
    {% if block.block_type != "detail" or render_detail %}
      <section class="block-{{ block.block_type }}">
        {% include_block block %}
      </section>
      {% block detail_link %}{% endblock %}
    {% endif %}
  {% endfor %}
</article>
