# Generated by Django 4.1.7 on 2023-03-05 21:36

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cast", "0039_blog_noindex"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blog",
            name="noindex",
            field=models.BooleanField(
                default=False,
                help_text="Whether to add a noindex meta tag to this page and all subpages excluding them from search engines.",
                verbose_name="noindex",
            ),
        ),
    ]
