# Generated by Django 4.1.7 on 2023-03-05 21:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("wagtailcore", "0083_workflowcontenttype"),
        ("cast", "0040_alter_blog_noindex"),
    ]

    operations = [
        migrations.CreateModel(
            name="TemplateBaseDirectory",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "name",
                    models.CharField(
                        choices=[("bootstrap4", "Bootstrap 4"), ("plain", "Just HTML")],
                        default="bootstrap4",
                        max_length=10,
                    ),
                ),
                (
                    "site",
                    models.OneToOneField(
                        editable=False, on_delete=django.db.models.deletion.CASCADE, to="wagtailcore.site"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
