/* These styles are generated from project.scss. */

.alert-debug {
  color: black;
  background-color: white;
  border-color: #d6e9c6;
}

.alert-error {
  color: #b94a48;
  background-color: #f2dede;
  border-color: #eed3d7;
}

/* This is a fix for the bootstrap4 alpha release */
@media (max-width: 47.9em) {
  .navbar-nav .nav-item {
    float: none;
    width: 100%;
    display: inline-block;
  }

  .navbar-nav .nav-item + .nav-item {
    margin-left: 0;
  }

  .nav.navbar-nav.pull-xs-right {
    float: none !important;
  }
}

/* Display django-debug-toolbar.
   See https://github.com/django-debug-toolbar/django-debug-toolbar/issues/742
   and https://github.com/pydanny/cookiecutter-django/issues/317
*/
[hidden][style="display: block;"] {
  display: block !important;
}

/* pygments syntax highlighing */
/* .highlight .hll { background-color: #ffffcc } */
/* .highlight  { background: #f8f8f8; } */
.highlight .c { color: #408080; font-style: italic } /* Comment */
.highlight .err { border: 1px solid #FF0000 } /* Error */
.highlight .k { color: #008000; font-weight: bold } /* Keyword */
.highlight .o { color: #666666 } /* Operator */
.highlight .ch { color: #408080; font-style: italic } /* Comment.Hashbang */
.highlight .cm { color: #408080; font-style: italic } /* Comment.Multiline */
.highlight .cp { color: #BC7A00 } /* Comment.Preproc */
.highlight .cpf { color: #408080; font-style: italic } /* Comment.PreprocFile */
.highlight .c1 { color: #408080; font-style: italic } /* Comment.Single */
.highlight .cs { color: #408080; font-style: italic } /* Comment.Special */
.highlight .gd { color: #A00000 } /* Generic.Deleted */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gr { color: #FF0000 } /* Generic.Error */
.highlight .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.highlight .gi { color: #00A000 } /* Generic.Inserted */
.highlight .go { color: #888888 } /* Generic.Output */
.highlight .gp { color: #000080; font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.highlight .gt { color: #0044DD } /* Generic.Traceback */
.highlight .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
.highlight .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
.highlight .kn { color: #008000; font-weight: bold } /* Keyword.Namespace */
.highlight .kp { color: #008000 } /* Keyword.Pseudo */
.highlight .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
.highlight .kt { color: #B00040 } /* Keyword.Type */
.highlight .m { color: #666666 } /* Literal.Number */
.highlight .s { color: #BA2121 } /* Literal.String */
.highlight .na { color: #7D9029 } /* Name.Attribute */
.highlight .nb { color: #008000 } /* Name.Builtin */
.highlight .nc { color: #0000FF; font-weight: bold } /* Name.Class */
.highlight .no { color: #880000 } /* Name.Constant */
.highlight .nd { color: #AA22FF } /* Name.Decorator */
.highlight .ni { color: #999999; font-weight: bold } /* Name.Entity */
.highlight .ne { color: #D2413A; font-weight: bold } /* Name.Exception */
.highlight .nf { color: #0000FF } /* Name.Function */
.highlight .nl { color: #A0A000 } /* Name.Label */
.highlight .nn { color: #0000FF; font-weight: bold } /* Name.Namespace */
.highlight .nt { color: #008000; font-weight: bold } /* Name.Tag */
.highlight .nv { color: #19177C } /* Name.Variable */
.highlight .ow { color: #AA22FF; font-weight: bold } /* Operator.Word */
.highlight .w { color: #bbbbbb } /* Text.Whitespace */
.highlight .mb { color: #666666 } /* Literal.Number.Bin */
.highlight .mf { color: #666666 } /* Literal.Number.Float */
.highlight .mh { color: #666666 } /* Literal.Number.Hex */
.highlight .mi { color: #666666 } /* Literal.Number.Integer */
.highlight .mo { color: #666666 } /* Literal.Number.Oct */
.highlight .sa { color: #BA2121 } /* Literal.String.Affix */
.highlight .sb { color: #BA2121 } /* Literal.String.Backtick */
.highlight .sc { color: #BA2121 } /* Literal.String.Char */
.highlight .dl { color: #BA2121 } /* Literal.String.Delimiter */
.highlight .sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */
.highlight .s2 { color: #BA2121 } /* Literal.String.Double */
.highlight .se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
.highlight .sh { color: #BA2121 } /* Literal.String.Heredoc */
.highlight .si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */
.highlight .sx { color: #008000 } /* Literal.String.Other */
.highlight .sr { color: #BB6688 } /* Literal.String.Regex */
.highlight .s1 { color: #BA2121 } /* Literal.String.Single */
.highlight .ss { color: #19177C } /* Literal.String.Symbol */
.highlight .bp { color: #008000 } /* Name.Builtin.Pseudo */
.highlight .fm { color: #0000FF } /* Name.Function.Magic */
.highlight .vc { color: #19177C } /* Name.Variable.Class */
.highlight .vg { color: #19177C } /* Name.Variable.Global */
.highlight .vi { color: #19177C } /* Name.Variable.Instance */
.highlight .vm { color: #19177C } /* Name.Variable.Magic */
.highlight .il { color: #666666 } /* Literal.Number.Integer.Long */

.cast-image {
    width: 100%;
    height: auto;
    max-height: 740px;
    object-fit: contain;
}

.cast-gallery-container {
    padding: 30px 0px 0px 0px;
}

.cast-gallery-content {
    background-color:rgba(0, 0, 0, 0);
}

.cast-gallery-body {
    display: flex;
    align-items: center;
    justify-content: center;
    background-color: rgba(255, 255, 255, 0.8);
}

.cast-gallery-header {
    background-color: rgba(255, 255, 255, 1);
    border-radius: .3rem .3rem 0rem 0rem
}

.cast-gallery-footer {
    background-color:rgba(255, 255, 255, 1);
    border-radius: 0rem 0rem .3rem .3rem
}

.cast-gallery-lg {
    min-width: 71%;
}

.cast-gallery-thumbnail {
    max-width: 10em;
    max-height: 10em;
    height: 5em;
    margin-bottom: 5px;
}

.cast-gallery-preview {
    position: relative;
    float: left;
    margin-right: 10px;
    margin-bottom: 10px;
    margin-top: 10px;
}

.cast-gallery-progress-bar {
    width: 100%;
}

.cast-video {
    max-height: 800px;
    max-width: 100%;
}

.cast-feeds-container {
    display: flex;
    flex-flow: row wrap;
    justify-content: space-around;
}

.cast-feed-item {
}

.cast-feed-icon {
}

.cast-post-list-header {
    display: flex;
    justify-content: space-between;
}

.cast-date-facet-container {
    display: flex;
    flex-wrap: wrap;
    height: auto;
}

.cast-date-facet-item {
    margin-right: 10px;
    margin-left: 10px;
}

.invalid-feedback {
    /* Workaround for django-fluentcomments error messages not showing up
    See https://github.com/ephes/django-cast/issues/107
    */
    display: block;
}
