# Generated by Django 3.2.6 on 2021-08-20 09:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailimages", "0023_add_choose_permissions"),
        ("cast", "0012_alter_post_images"),
    ]

    operations = [
        migrations.AlterField(
            model_name="gallery",
            name="images",
            field=models.ManyToManyField(to="wagtailimages.Image"),
        ),
    ]
