# Generated by Django 3.1.2 on 2020-10-24 06:13

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0052_pagelogentry"),
        ("cast", "0004_homepage_alias_for_page"),
    ]

    operations = [
        migrations.AlterField(
            model_name="homepage",
            name="alias_for_page",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Make this page an alias for another page, redirecting to it with a non permanent redirect.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="aliases",
                to="wagtailcore.page",
                verbose_name="Redirect to another page",
            ),
        ),
    ]
