# Generated by Django 4.1.7 on 2023-03-12 21:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cast", "0042_blog_template_base_dir"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blog",
            name="template_base_dir",
            field=models.CharField(
                blank=True,
                choices=[("bootstrap4", "Bootstrap 4"), ("plain", "Just HTML")],
                default=None,
                max_length=10,
                null=True,
            ),
        ),
    ]
