{% extends "wagtailadmin/widgets/chooser.html" %}
{% block chooser_class %}video-chooser{% endblock %}

{% block chosen_state_view %}
    <span class="title">{{ title }}</span>
{% endblock %}

{% block edit_chosen_item_url %}{{ edit_url }}{% endblock %}
