# Generated by Django 4.1.4 on 2023-02-02 21:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0029_add_metadata_field_to_audio"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="post",
            name="pub_date",
        ),
    ]
