# Generated by Django 4.1.4 on 2023-02-04 14:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0034_remove_old_podcast_fields_from_blog"),
    ]

    operations = [
        migrations.RenameField(
            model_name="podcast",
            old_name="new_explicit",
            new_name="explicit",
        ),
        migrations.RenameField(
            model_name="podcast",
            old_name="new_itunes_artwork",
            new_name="itunes_artwork",
        ),
        migrations.RenameField(
            model_name="podcast",
            old_name="new_itunes_categories",
            new_name="itunes_categories",
        ),
        migrations.RenameField(
            model_name="podcast",
            old_name="new_keywords",
            new_name="keywords",
        ),
    ]
