# Generated by Django 4.1.5 on 2023-01-21 06:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0027_episode"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="post",
            name="block",
        ),
        migrations.RemoveField(
            model_name="post",
            name="explicit",
        ),
        migrations.RemoveField(
            model_name="post",
            name="keywords",
        ),
        migrations.RemoveField(
            model_name="post",
            name="podcast_audio",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_block",
            new_name="block",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_explicit",
            new_name="explicit",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_keywords",
            new_name="keywords",
        ),
        migrations.RenameField(
            model_name="episode",
            old_name="new_podcast_audio",
            new_name="podcast_audio",
        ),
    ]
