{% load wagtailimages_tags %}
<a href="{{ value.file.url }}">
  <picture>
    <source
      srcset="{{ value.regular.srcset.avif }}"
      type="image/avif"
      sizes="(max-width: 600px) 555px, {{ value.regular.sizes }}"
    >
    <img
      class="cast-image"
      alt="{{ value.default_alt_text }}"
      src="{{ value.regular.src.jpeg }}"
      srcset="{{ value.regular.srcset.jpeg }}"
      sizes="(max-width: 600px) 555px, {{ value.regular.sizes }}"
      width="{{ value.regular.width }}"
      height="{{ value.regular.height }}"
    >
  </picture>
</a>
