{% if value.poster %}
  <video class="cast-video" preload="auto" controls poster="{{ value.poster.url}}">
    <source src="{{ value.original.url }}" type="{{ value.get_mime_type }}">
    your browser does not support the video tag
  </video>
{% else %}
  <video class="cast-video" preload="auto" controls>
    <source src="{{ value.original.url }}" type="{{ value.get_mime_type }}">
    your browser does not support the video tag
  </video>
{% endif %}
