{% extends "wagtailadmin/base.html" %}
{% load i18n %}
{% load wagtailimages_tags wagtailadmin_tags %}
{% block titletag %}{% blocktrans with title=video.title %}Editing {{ title }}{% endblocktrans %}{% endblock %}

{% block extra_js %}
  {{ block.super }}
  {{ form.media.js }}

  {% url 'wagtailadmin_tag_autocomplete' as autocomplete_url %}
  <script>
    $(function() {
      $('#id_tags').tagit({
        autocomplete: {source: "{{ autocomplete_url|addslashes }}"}
      });
    });
  </script>
{% endblock %}

{% block extra_css %}
  {{ block.super }}
  {{ form.media.css }}
{% endblock %}

{% block content %}
  {% trans "Editing" as editing_str %}
  {% include "wagtailadmin/shared/header.html" with title=editing_str subtitle=video.title icon="media" usage_object=video %}

  {% block form_row %}
    <div class="row row-flush nice-padding">

      <div class="col10 divider-after">
        <form action="{% url 'castvideo:edit' video.id %}" method="POST" enctype="multipart/form-data" novalidate>
          {% csrf_token %}
          <ul class="fields">
            {% for field in form %}
              {% if field.name == 'file' %}
                {% include "cast/wagtail/_file_field_as_li.html" %}
              {% elif field.name == 'thumbnail' %}
                {% include "cast/wagtail/_thumbnail_field_as_li.html" %}
              {% else %}
                {% include "wagtailadmin/shared/field_as_li.html" %}
              {% endif %}
            {% endfor %}
            <li>
              <input class="button" type="submit" value="{% trans 'Save' %}" />
              {% if user_can_delete %}
                <a href="{% url 'castvideo:delete' video.id %}" class="button button-secondary no">{% trans "Delete" %}</a>
              {% endif %}
            </li>
          </ul>
        </form>
      </div>
      <div class="col2">
        <dl>
          {% block media_stats %}
            {% if video.file %}
              <dt>{% trans "Filesize" %}</dt>
              <dd>{% if filesize %}{{ filesize|filesizeformat }}{% else %}{% trans "File not found" %}{% endif %}</dd>
            {% endif %}

          {% endblock %}
        </dl>
      </div>
    </div>
  {% endblock %}

  </div>
{% endblock %}
