# Generated by Django 4.1.4 on 2023-02-05 09:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0036_alter_blog_author"),
    ]

    operations = [
        migrations.AlterField(
            model_name="episode",
            name="block",
            field=models.BooleanField(
                default=False,
                help_text="Check to block this episode from iTunes because <br />its content might cause the entire show to be <br />removed from iTunes.",
            ),
        ),
        migrations.AlterField(
            model_name="episode",
            name="explicit",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "yes"), (2, "no"), (3, "clean")],
                default=1,
                help_text="``Clean`` will put the clean iTunes graphic by it.",
            ),
        ),
        migrations.AlterField(
            model_name="episode",
            name="keywords",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A comma-demlimitedlist of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="podcast",
            name="keywords",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A comma-delimited list of up to 12 words for iTunes\n            searches. Perhaps include misspellings of the title.",
                max_length=255,
                verbose_name="keywords",
            ),
        ),
        migrations.AlterField(
            model_name="post",
            name="comments_enabled",
            field=models.BooleanField(default=True, help_text="Whether comments are enabled for this post."),
        ),
    ]
