# Generated by Django 4.1.7 on 2023-03-12 21:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cast", "0041_templatebasedirectory"),
    ]

    operations = [
        migrations.AddField(
            model_name="blog",
            name="template_base_dir",
            field=models.CharField(
                choices=[("bootstrap4", "Bootstrap 4"), ("plain", "Just HTML")], default="bootstrap4", max_length=10
            ),
        ),
    ]
