# Generated by Django 4.1.5 on 2023-01-04 13:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0024_alter_homepage_body_alter_post_body"),
    ]

    operations = [
        migrations.AddField(
            model_name="spamfilter",
            name="performance",
            field=models.JSONField(default=dict, verbose_name="Spamfilter Performance Indicators"),
        ),
    ]
