# Generated by Django 4.1.4 on 2023-02-04 16:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0035_remove_new_prefix_podcast_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blog",
            name="author",
            field=models.CharField(
                blank=True,
                default=None,
                help_text="Freeform text that will be used in the feed.",
                max_length=255,
                null=True,
            ),
        ),
    ]
