# Generated by Django 4.2.4 on 2023-08-15 11:58

from django.db import migrations, models
import modelcluster.fields


class Migration(migrations.Migration):
    dependencies = [
        ("cast", "0048_added_visible_date_index_for_wagtail_api"),
    ]

    operations = [
        migrations.CreateModel(
            name="PostCategory",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(help_text="The name for this category", max_length=255, unique=True)),
                (
                    "slug",
                    models.SlugField(
                        help_text="A slug to identify posts by this category", unique=True, verbose_name="slug"
                    ),
                ),
            ],
            options={
                "verbose_name": "Post Category",
                "verbose_name_plural": "Post Categories",
                "ordering": ["name"],
            },
        ),
        migrations.AddField(
            model_name="post",
            name="categories",
            field=modelcluster.fields.ParentalManyToManyField(blank=True, to="cast.postcategory"),
        ),
    ]
