# Generated by Django 4.1.4 on 2023-02-04 13:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0033_add_new_podcast_model"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="blog",
            name="explicit",
        ),
        migrations.RemoveField(
            model_name="blog",
            name="itunes_artwork",
        ),
        migrations.RemoveField(
            model_name="blog",
            name="itunes_categories",
        ),
        migrations.RemoveField(
            model_name="blog",
            name="keywords",
        ),
    ]
