# Generated by Django 4.1.8 on 2023-04-09 06:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("cast", "0045_alter_blog_template_base_dir_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="episode",
            name="podcast_audio",
            field=models.ForeignKey(
                blank=True,
                help_text="The audio file for this episode -if this is not set,the episode will not be included in the feed.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="episodes",
                to="cast.audio",
            ),
        ),
    ]
