{% extends "./post.html" %}

{% block meta %}
  {{ block.super }}
  {% if episode.podcast_audio %}
    <!-- Twitter Player Card -->
    <!-- Implement in your own template for now -->
  {% endif %}
{% endblock %}
