# Generated by Django 3.2.7 on 2021-10-01 09:21

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0020_auto_20210926_1556"),
    ]

    operations = [
        migrations.CreateModel(
            name="SpamFilter",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                ("name", models.CharField(max_length=128, unique=True)),
                ("model", models.JSONField(default=dict, verbose_name="Spamfilter Model")),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
