{% load static wagtailuserbar %}
{% load static i18n %}
<!DOCTYPE html>
<html lang="en">
  <head>
    {% block meta %}
      <meta charset="utf-8">
      <meta http-equiv="x-ua-compatible" content="ie=edge">
      <meta name="viewport" content="width=device-width, initial-scale=1.0">
      <meta name="description" content="">
      <meta name="author" content="">
    {% endblock %}

    <title>{% block title %}Wagtail/Django-Cast Example{% endblock title %}</title>

    <!-- HTML5 shim, for IE6-8 support of HTML5 elements -->
    <!--[if lt IE 9]>
      <script src="https://cdnjs.cloudflare.com/ajax/libs/html5shiv/3.7.3/html5shiv.min.js"></script>
    <![endif]-->

    <link rel="icon" href="{% static 'images/favicons/favicon.ico' %}">

    {% block css %}

      <!-- Latest compiled and minified Bootstrap CSS -->
      <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap@4.6.2/dist/css/bootstrap.min.css" integrity="sha384-xOolHFLEh07PJGoPkLv1IbcEPTNtaed2xpHsD9ESMhqIYd0nLMwNLD69Npy4HI+N" crossorigin="anonymous">


      <!-- Your stuff: Third-party CSS libraries go here -->

      <!-- This file stores project-specific CSS -->

      <link href="{% static 'css/project.css' %}" rel="stylesheet">


    {% endblock %}

  </head>
  <body>

    <div class="m-b-1">
      <nav class="navbar navbar-expand-md navbar-light bg-light">
        <button class="navbar-toggler navbar-toggler-right" type="button" data-toggle="collapse" data-target="#navbarSupportedContent" aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
          <span class="navbar-toggler-icon"></span>
        </button>
        <a class="navbar-brand" href="/">Wagtail/Django-Cast Example</a>

        <div class="collapse navbar-collapse" id="navbarSupportedContent">
          <ul class="navbar-nav mr-auto">
            <li class="nav-item active">
              <a class="nav-link" href="/">Home <span class="sr-only">(current)</span></a>
            </li>
            <li class="nav-item">
              <a class="nav-link" href="{% url 'about' %}">About</a>
            </li>
          </ul>
        </div>
      </nav>

    </div>
    <div class="container">

      {% if messages %}
        {% for message in messages %}
          <div class="alert {% if message.tags %}alert-{{ message.tags }}{% endif %}">{{ message }}</div>
        {% endfor %}
      {% endif %}

      {% block content %}
        <p>Hallo Dominik :)</p>
      {% endblock content %}

    </div> <!-- /container -->

    {% block modal %}{% endblock modal %}

    <!-- Le javascript
    ================================================== -->
    <!-- Placed at the end of the document so the pages load faster -->
    {% block javascript %}

      <!-- Required by Bootstrap -->
      <script
        src="https://code.jquery.com/jquery-3.6.3.min.js"
        integrity="sha256-pvPw+upLPUjgMXY0G+8O0xUf+/Im1MZjXxxgOcBQBXU="
        crossorigin="anonymous">
      </script>
      <script src="https://cdn.jsdelivr.net/npm/bootstrap@4.6.2/dist/js/bootstrap.min.js" integrity="sha384-+sLIOodYLS7CIrQpBjl+C7nPvqq+FbNUBDunl/OZv93DB7Ln/533i8e/mZXLi/P+" crossorigin="anonymous"></script>

      <!-- Your stuff: Third-party javascript libraries go here -->


      <!-- place project specific Javascript in this file -->

      <script src="{% static 'js/project.js' %}"></script>


    {% endblock javascript %}
  </body>
</html>
