# Generated by Django 4.1.5 on 2023-01-28 05:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cast", "0028_rename_and_drop_itune_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="audio",
            name="data",
            field=models.JSONField(blank=True, default=dict, verbose_name="Metadata"),
        ),
    ]
