{% extends "wagtailadmin/shared/field.html" %}
{% load i18n %}
{% block form_field %}

    {% if media.thumbnail %}
        <a href="{{ media.thumbnail.url }}" class="icon icon-image">{{ media.thumbnail_filename }}</a><br /><br />
    {% endif %}

    {% trans "Change thumbnail file:" %}
    {{ field }}
{% endblock %}
